package com.jpattern.orm;

import java.util.HashMap;
import java.util.Map;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.crud.IOrmCRUDQuery;
import com.jpattern.orm.crud.OrmCRUDQueryGenerator;
import com.jpattern.orm.dialect.DefaultDialect;
import com.jpattern.orm.dialect.IDialect;
import com.jpattern.orm.exception.OrmConfigurationException;
import com.jpattern.orm.exception.OrmNotFoundException;
import com.jpattern.orm.generator.CojenPersistorGenerator;
import com.jpattern.orm.logger.OrmLogger;
import com.jpattern.orm.mapper.ClassMapperBuilder;
import com.jpattern.orm.mapper.ClassTableMapBuilder;
import com.jpattern.orm.mapper.IClassMapper;
import com.jpattern.orm.mapper.TableMap;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.session.NullSessionProvider;
import com.jpattern.orm.session.Session;

/**
 * 
 * @author Francesco Cina'
 *
 * 26/ago/2011
 */
public final class JPOrm implements IJPOrm {

	private static final Map<String, IJPOrm> JPORMS_MAP = new HashMap<String, IJPOrm>(); 

	private ISessionProvider sessionProvider;
	private IOrmClassToolMap ormClassToolMap = new OrmClassToolMap();
	private ILogger logger = OrmLogger.getOrmLogger(this.getClass());
	private final IDialect dialect;
	private final String jpOrmName;

	/**
	 * Create a new instance of JPOrm using the DefaultDialect (compatible with Oracle and H2 dbs).
	 * This instance can be obtained everywhere in the application using 
	 * the static method get(String jpOrmName)
	 * @param jpOrmName
	 * @param sessionProvider
	 */
	public JPOrm(String jpOrmName, ISessionProvider sessionProvider) {
		this(jpOrmName, sessionProvider, new DefaultDialect());
	}
	
	/**
	 * Create a new instance of JPOrm.
	 * This instance can be obtained everywhere in the application using 
	 * the static method get(String jpOrmName)
	 * @param jpOrmName
	 * @param sessionProvider
	 * @param dialect
	 */
	public JPOrm(String jpOrmName, ISessionProvider sessionProvider, IDialect dialect) {
		this.jpOrmName = jpOrmName;
		this.sessionProvider = sessionProvider;
		this.dialect = dialect;
		JPORMS_MAP.put(jpOrmName, this);
	}

	@Override
	public final ISession session() {
		return new Session(ormClassToolMap, sessionProvider.getSessionStrategy() );
	}
	
	/**
	 * This permits to obtain everywhere in the application a IJPorm instance
	 * by the name. 
	 * @param jpOrmName the name of the JPOrm instance
	 * @return
	 * @throws OrmNotFoundException thrown if no JPOrm instance are found with name jpOrmName
	 */
	public static final IJPOrm get(String jpOrmName) throws OrmNotFoundException {
		if (JPORMS_MAP.containsKey(jpOrmName)) {
			return JPORMS_MAP.get(jpOrmName);
		}
		throw new OrmNotFoundException( "No JPOrm instance found with name " + jpOrmName );
	}

	@Override
	public final String getName() {
		return jpOrmName;
	}

	@Override
	public <T> void register(Class<T> clazz) throws OrmConfigurationException {
		register(clazz, true);
	}
	
	@Override
	public <T> void register(Class<T> clazz, boolean enableDBScan) throws OrmConfigurationException {
		logger.info("register", "register new class: " + clazz.getName());
		try {
			TableMap tableMap = new TableMap();
			new ClassTableMapBuilder(clazz, tableMap).updateTableMap();
			sessionProvider.updateTableMap(tableMap , enableDBScan);
			IClassMapper<T> classMapper = new ClassMapperBuilder<T>(clazz, tableMap).generate();
			IOrmPersistor<T> ormPersistor = new CojenPersistorGenerator<T>(classMapper).generate();
			IOrmCRUDQuery ormCRUDQuery = new OrmCRUDQueryGenerator<T>(dialect, classMapper).generate();
			IOrmClassTool<T> ormClassTool = new OrmClassTool<T>(classMapper, ormPersistor, ormCRUDQuery);
			ormClassToolMap.put(clazz, ormClassTool);
		} catch (Exception e) {
			throw new OrmConfigurationException(e);
		}
	}

	@Override
	public void destory() {
		JPORMS_MAP.remove(jpOrmName);
		sessionProvider = new NullSessionProvider();
		ormClassToolMap = new NullOrmClassToolMap();
	}

}
